#!/usr/bin/perl

use strict;
use warnings;

use Test::More;

use_ok( 'App::Devel::MAT::Explorer::GTK::Resources' );
use_ok( 'App::Devel::MAT::Explorer::GTK::SVDetail' );
use_ok( 'App::Devel::MAT::Explorer::GTK::Widgets' );

done_testing;
